package example;

import java.io.IOException;
import java.net.InetSocketAddress;

import org.jboss.logging.Logger;
import org.jboss.remoting3.Channel;
import org.jboss.remoting3.CloseHandler;
import org.jboss.remoting3.Endpoint;
import org.jboss.remoting3.OpenListener;
import org.jboss.remoting3.Remoting;
import org.jboss.remoting3.remote.RemoteConnectionProviderFactory;
import org.jboss.remoting3.security.SimpleServerAuthenticationProvider;
import org.jboss.remoting3.spi.NetworkServerProvider;
import org.xnio.FutureResult;
import org.xnio.OptionMap;
import org.xnio.Options;
import org.xnio.Sequence;

public class Server {
	private static final Logger LOGGER = Logger.getLogger(Server.class);
	
	private Endpoint endpoint;
	
	public Server() throws Exception {
		endpoint = Remoting.createEndpoint("test", OptionMap.EMPTY);
		endpoint.addConnectionProvider("remote", new RemoteConnectionProviderFactory(), OptionMap.create(Options.SSL_ENABLED, Boolean.FALSE));
		NetworkServerProvider networkServerProvider = endpoint.getConnectionProviderInterface("remote",NetworkServerProvider.class);
		SimpleServerAuthenticationProvider provider = new SimpleServerAuthenticationProvider();
		provider.addUser("bob", "test", "pass".toCharArray());
		networkServerProvider.createServer(new InetSocketAddress("localhost", 4447),
				OptionMap.create(Options.SASL_MECHANISMS,
				Sequence.of("CRAM-MD5")), provider, null);
		
		final FutureResult<Channel> passer = new FutureResult<Channel>();
        endpoint.registerService("org.jboss.test", new OpenListener() {
            public void channelOpened(final Channel channel) {
                passer.setResult(channel);
            }

            public void registrationTerminated() {
            }
        }, OptionMap.EMPTY);
		
		endpoint.addCloseHandler(new CloseHandler<Endpoint>() {
			public void handleClose(final Endpoint closed,
					final IOException exception) {
				LOGGER.info("shutdown complete!");
			}
		});
	}
	
	public void close() throws Exception {
		endpoint.closeAsync();
	}
}
